import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useState, useEffect } from 'react';
import isDeepEqual from 'fast-deep-equal/react';
import Grid from '@mui/material/Grid';
import TextField from '@mui/material/TextField';
import { getDateFormatInfo } from '../../helpers/date-format-utils';
import { getCurrencyOptions } from '../../field/Currency/currency-utils';
import { getComponentFromMap } from '../../../bridge/helpers/sdk_component_map';
import './CaseSummaryFields.css';
import { format } from '../../helpers/formatters';
export default function CaseSummaryFields(props) {
    // Get emitted components from map (so we can get any override that may exist)
    const Operator = getComponentFromMap('Operator');
    const { status, showStatus, theFields } = props;
    const [theFieldsToRender, setFieldsToRender] = useState([]);
    const [theFieldsAsGridItems, setFieldsAsGridItems] = useState([]);
    function getFieldValue(field) {
        const fieldTypeLower = field.type.toLowerCase();
        if (field.config.value === null || field.config.value === '') {
            // Special handling for missing value
            // eslint-disable-next-line sonarjs/no-small-switch
            switch (fieldTypeLower) {
                case 'caseoperator':
                    return _jsx(Operator, { ...field.config });
                    break;
                default:
                    return (_jsx(TextField, { variant: 'standard', value: '---', label: field.config.label, InputProps: {
                            readOnly: true,
                            disableUnderline: true
                        } }));
            }
        }
        switch (fieldTypeLower) {
            case 'textinput':
            case 'decimal':
            case 'integer':
            case 'dropdown':
                return (_jsx(TextField, { variant: 'standard', value: field.config.value, label: field.config.label, InputProps: {
                        readOnly: true,
                        disableUnderline: true
                    } }));
            case 'checkbox': {
                const { caption, label, value, trueLabel, falseLabel } = field.config;
                const fieldLabel = label || caption;
                const fieldValue = value ? trueLabel : falseLabel;
                return (_jsx(TextField, { variant: 'standard', value: fieldValue, label: fieldLabel, InputProps: {
                        readOnly: true,
                        disableUnderline: true
                    } }));
            }
            case 'status':
                return (_jsx(TextField, { variant: 'standard', className: 'psdk-csf-status-style', value: field.config.value, label: field.config.label, InputProps: {
                        readOnly: true,
                        disableUnderline: true
                    } }));
            case 'phone': {
                const displayPhone = field.config.value !== '' ? field.config.value : '---';
                return (_jsx("a", { href: `tel:${displayPhone}`, children: _jsx(TextField, { variant: 'standard', value: field.config.value, label: field.config.label, InputProps: {
                            readOnly: true,
                            inputProps: {
                                style: { cursor: 'pointer' },
                                disableUnderline: true
                            }
                        } }) }));
            }
            case 'email': {
                const displayEmail = format(field.config.value, field.type);
                return (_jsx("a", { href: `mailto:${displayEmail}`, children: _jsx(TextField, { variant: 'standard', value: field.config.value, label: field.config.label, InputProps: {
                            readOnly: true,
                            disableUnderline: true,
                            inputProps: { style: { cursor: 'pointer' } }
                        } }) }));
            }
            case 'date':
            case 'datetime': {
                const theDateFormatInfo = getDateFormatInfo();
                // console.log(`theDateFormatInfo: ${theDateFormatInfo}`);
                const theFormat = fieldTypeLower === 'datetime' ? `${theDateFormatInfo.dateFormatStringLong} hh:mm a` : theDateFormatInfo.dateFormatStringLong;
                return (_jsx(TextField, { variant: 'standard', value: format(field.config.value, field.type, {
                        format: theFormat
                    }), label: field.config.label, InputProps: {
                        readOnly: true,
                        disableUnderline: true
                    } }));
            }
            case 'currency': {
                const theCurrencyOptions = getCurrencyOptions(field.config?.currencyISOCode);
                return (_jsx(TextField, { variant: 'standard', value: format(field.config.value, field.type, theCurrencyOptions), label: field.config.label, InputProps: {
                        readOnly: true,
                        disableUnderline: true
                    } }));
            }
            case 'boolean':
            case 'userreference':
                return (_jsx(TextField, { variant: 'standard', value: format(field.config.value, field.type), label: field.config.label, InputProps: {
                        readOnly: true,
                        disableUnderline: true
                    } }));
            case 'caseoperator':
                return _jsx(Operator, { ...field.config });
            default:
                return (_jsxs("span", { children: [field.type.toLowerCase(), " ", field.config.value] }));
        }
    }
    // Whenever theFieldsToRender changes, update theFieldsAsGridItems that's used during render
    useEffect(() => {
        const arGridItems = theFieldsToRender.map((field) => {
            // display the field when either visibility property doesn't exist or is true(if exists)
            if (field.config.visibility === undefined || field.config.visibility === true) {
                return (_jsx(Grid, { item: true, xs: 6, children: getFieldValue(field) }, field.config.label));
            }
            return null;
        });
        setFieldsAsGridItems(arGridItems);
    }, [theFieldsToRender]);
    const theFieldsModifiable = theFields;
    // Special Case: if showStatus is true, splice the status value to be 2nd in theFields
    //  if it's not already there
    if (showStatus && theFields?.[1].type !== 'status') {
        const oStatus = {
            type: 'status',
            config: { value: status, label: 'Status' }
        };
        const count = theFieldsModifiable.length;
        if (count < 2) {
            theFieldsModifiable.push(oStatus);
        }
        else {
            theFieldsModifiable.splice(1, 0, oStatus);
        }
    }
    // At this point, we know what fields we want to render...
    //  So, update our state if it's changed
    if (!isDeepEqual(theFieldsToRender, theFieldsModifiable)) {
        setFieldsToRender(theFieldsModifiable);
    }
    return (_jsx(Grid, { container: true, className: 'psdk-case-summary-fields', children: theFieldsAsGridItems }));
}
//# sourceMappingURL=CaseSummaryFields.js.map